@@ -91,7 +91,7 @@ class Hoe
   include Rake::DSL if defined?(Rake::DSL)
 
   # duh
-  VERSION = '3.0.3'
+  VERSION = "3.8.0"
 
   @@plugins = [:clean, :debug, :deps, :flay, :flog, :newb, :package,
                :publish, :gemcutter, :signing, :test]
@@ -182,6 +182,13 @@ class Hoe
 
   attr_accessor :history_file
 
+  ##
+  # Optional: An array containing the license(s) under which this gem is released.
+  #
+  # Warns and defaults to "MIT" if not set.
+
+  attr_accessor :licenses
+
   ##
   # *MANDATORY*: The name of the release.
   #
@@ -200,9 +207,12 @@ class Hoe
   attr_accessor :readme_file
 
   ##
-  # Optional: The name of the rubyforge project. [default: name.downcase]
+  # Optional: The name of the group authoring the project. [default: name.downcase]
 
-  attr_accessor :rubyforge_name
+  attr_accessor :group_name
+
+  alias :rubyforge_name  :group_name  # Deprecated. Use #group_name.
+  alias :rubyforge_name= :group_name= # Deprecated. Use #group_name=.
 
   ##
   # The Gem::Specification.
@@ -242,12 +252,17 @@ class Hoe
   # README.txt.
   #
 
-  attr_accessor :url
+  def url
+    warn "NOTE: Hoe#url is deprecated, use urls. It will be removed on or after 2012-06-01."
+    warn "Used from #{caller.first}"
+    @url
+  end
 
-  require 'rubygems/deprecate'
-  extend Gem::Deprecate
-  deprecate :url,  :urls,  2012, 6
-  deprecate :url=, :urls=, 2012, 6
+  def url=o # :nodoc:
+    warn "NOTE: Hoe#url= is deprecated, use urls=. It will be removed on or after 2012-06-01."
+    warn "Used from #{caller.first}"
+    @url=o
+  end
 
   ##
   # Optional: The urls of the project. This can be an array or
@@ -317,7 +332,7 @@ class Hoe
   end
 
   ##
-  # Normalize a project name into the project, file, and klass names that
+  # Normalize a project name into the project, file, klass and test names that
   # follow Ruby package naming guidelines.
   #
   # Project names are lowercase with _ separating package parts and -
@@ -327,14 +342,17 @@ class Hoe
   # extension parts.  net-http-persistent becomes net/http/persistent.
   #
   # Klass names are CamelCase with :: separating extension parts.
+  #
+  # Test klass names are same as Klass with Test prepended to each part.
 
   def self.normalize_names project # :nodoc:
-    project   = project.gsub(/([A-Z])/, '_\1').downcase.sub(/^_/, '')
-    klass     = project.gsub(/(?:^|_)([a-z])/) { $1.upcase }
-    klass     = klass.  gsub(/(?:^|-)([a-z])/) { "::#{$1.upcase}" }
-    file_name = project.gsub(/-/, '/')
+    project    = project.gsub(/([A-Z])/, '_\1').downcase.sub(/^_/, '')
+    klass      = project.gsub(/(?:^|_)([a-z])/) { $1.upcase }
+    klass      = klass.  gsub(/(?:^|-)([a-z])/) { "::#{$1.upcase}" }
+    test_klass = klass.  gsub(/(^|::)([A-Z])/) { "#{$1}Test#{$2}" }
+    file_name  = project.gsub(/-/, '/')
 
-    return project, file_name, klass
+    return project, file_name, klass, test_klass
   end
 
   ##
@@ -401,6 +419,9 @@ class Hoe
     activate_plugin_deps
   end
 
+  ##
+  # Run all initialize_* methods for plugins
+
   def initialize_plugins
     Hoe.plugins.each do |plugin|
       msg = "initialize_#{plugin}"
@@ -409,6 +430,9 @@ class Hoe
     end
   end
 
+  ##
+  # Run all activate_*_deps methods for plugins
+
   def activate_plugin_deps
     Hoe.plugins.each do |plugin|
       msg = "activate_#{plugin}_deps"
@@ -417,6 +441,14 @@ class Hoe
     end
   end
 
+  ##
+  #  Specify a license for your gem.
+  #  Call it multiple times if you are releasing under multiple licenses.
+  #
+  def license name
+    self.licenses << name
+  end
+
   ##
   # Add a dependency declaration to your spec. Pass :dev to
   # +type+ for developer dependencies.
@@ -501,7 +533,7 @@ class Hoe
                                else
                                  raise "unknown urls format: #{urls.inspect}"
                                end
-      s.rubyforge_project    = rubyforge_name
+      s.rubyforge_project    = group_name
       s.description          = description
       s.files                = manifest
       s.executables          = s.files.grep(/^bin/) { |f| File.basename(f) }
@@ -509,7 +541,7 @@ class Hoe
       s.require_paths        = dirs unless dirs.empty?
       s.rdoc_options         = ['--main', readme_file]
       s.post_install_message = post_install_message
-      s.test_files           = Dir[*self.test_globs]
+      s.test_files           = Dir[*self.test_globs].uniq
 
       missing "Manifest.txt" if s.files.empty?
 
@@ -520,11 +552,25 @@ class Hoe
         s.author  = author
       end
 
-      s.extra_rdoc_files += s.files.grep(/(txt|rdoc)$/)
+      s.extra_rdoc_files += s.files.grep(/\.(txt|rdoc|md)$/)
       s.extra_rdoc_files.reject! { |f| f =~ %r%^(test|spec|vendor|template|data|tmp)/% }
       s.extra_rdoc_files += @extra_rdoc_files
     end
 
+    check_for_version
+
+    if licenses.empty?
+      warn "Defaulting gemspec to MIT license."
+      warn "Call license in hoe spec to change."
+      license "MIT"
+    end
+
+    spec.licenses = licenses
+
+    run_spec_extras
+  end
+
+  def check_for_version # :nodoc:
     unless self.version then
       version    = nil
       version_re = /VERSION += +([\"\'])([\d][\w\.]+)\1/
@@ -544,9 +590,11 @@ class Hoe
         warn "   or fix your Manifest.txt"
       end
     end
+  end
 
+  def run_spec_extras # :nodoc:
     # Do any extra stuff the user wants
-    spec_extras.each do |msg, val|
+    self.spec_extras.each do |msg, val|
       case val
       when Proc
         val.call spec.send(msg)
@@ -568,7 +616,7 @@ class Hoe
   # Returns true if the gem +name+ is installed.
 
   def have_gem? name
-    Gem::Specification.find_by_name name
+    Gem::Specification.find_by_name name.to_s
   rescue Gem::LoadError
     false
   end
@@ -588,21 +636,23 @@ class Hoe
     self.extra_deps           = []
     self.extra_dev_deps       = []
     self.extra_rdoc_files     = []
+    self.licenses             = []
     self.post_install_message = nil
-    self.rubyforge_name       = name.downcase
+    self.group_name           = name.downcase
     self.spec                 = nil
     self.spec_extras          = {}
     self.summary              = nil
     self.summary_sentences    = 1
-    self.test_globs           = ['test/**/test_*.rb']
+    self.test_globs           = ['test/**/{test,spec}_*.rb',
+                                 'test/**/*_{test,spec}.rb']
 
     if manifest = read_manifest then
       self.readme_file  = manifest.grep(/^README\./).first
       self.history_file = manifest.grep(/^History\./).first
     end
 
-    self.history_file ||= "History.txt"
-    self.readme_file  ||= "README.txt"
+    self.history_file ||= Dir.glob("History.{txt,md}").first
+    self.readme_file  ||= Dir.glob("README.{txt,md}").first
 
     abort "Hoe.new {...} removed. Switch to Hoe.spec." if block_given?
   end
@@ -657,7 +707,9 @@ class Hoe
   # should update the readme.
 
   def parse_urls text
-    lines = text.gsub(/^\* /, '').split(/\n/).grep(/\S+/)
+
+    lines = text.gsub(/^\* /, '').delete("<>").split(/\n/).grep(/\S+/)
+
     if lines.first =~ /::/ then
       Hash[lines.map { |line| line.split(/\s*::\s*/) }]
     else
@@ -773,24 +825,26 @@ class Hoe
   ##
   # Declare that your gem requires a specific rubygems version. Last one wins.
 
-  def require_rubygems_version version
-    spec_extras[:required_rubygems_version] = version
+  def require_rubygems_version *versions
+    spec_extras[:required_rubygems_version] = versions
   end
 
   ##
   # Declare that your gem requires a specific ruby version. Last one wins.
 
-  def require_ruby_version version
-    spec_extras[:required_ruby_version] = version
+  def require_ruby_version *versions
+    spec_extras[:required_ruby_version] = versions
   end
 
   ##
-  # Provide a linear degrading value from n to m over start to finis dates.
+  # Provide a linear degrading value from n to m over start to finis
+  # dates. If not provided, start and finis will default to 1/1 and
+  # 12/31 of the current year.
 
-  def timebomb n, m, finis = '2010-04-01', start = '2009-03-14'
+  def timebomb n, m, finis = nil, start = nil
     require 'time'
-    finis = Time.parse finis
-    start = Time.parse start
+    finis = Time.parse(finis || "#{Time.now.year}-12-31")
+    start = Time.parse(start || "#{Time.now.year}-01-01")
     rest  = (finis - Time.now)
     full  = (finis - start)
 
@@ -846,6 +900,6 @@ class File
   end
 end
 
-def Gem.bin_wrapper name # HACK
+def Gem.bin_wrapper name # :nodoc: HACK
   File.join Gem.bindir, Gem.default_exec_format % name
 end unless Gem.respond_to? :bin_wrapper
