@@ -2,12 +2,13 @@ require 'rubygems'
 require 'minitest/autorun'
 require 'hoe'
 
-class TestHoePublish < MiniTest::Unit::TestCase
+class TestHoePublish < Minitest::Test
   def setup
     @hoe = Hoe.spec 'blah' do
       self.version = '1.0'
 
       developer 'author', ''
+      license 'MIT'
     end
   end
 
